package com.gic.enterprise.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.WmMemberCardConfigDTO;
import com.gic.enterprise.dto.WmOrderConfigDTO;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.wm.*;
import com.gic.enterprise.qo.wm.CouponConfigQO;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 微盟商城店铺
 * @ClassName: WmStoreApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/4/13 10:47 AM 
 */
public interface WmStoreApiService {

    /**
     * 新增
     * @Title: saveWmStore 
     * @Description:
     * @author guojuxing
     * @param wmStoreDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveWmStore(WmStoreDTO wmStoreDTO);

    /**
     * 修改
     * @Title: modifyWmStore 
     * @Description:
     * @author guojuxing
     * @param wmStoreDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> modifyWmStore(WmStoreDTO wmStoreDTO);

    /**
     * 微盟商城店铺授权
     * @Title: authWmStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 店铺名称
     */
    ServiceResponse<String> authWmStore(Integer wmMallStoreId);

    ServiceResponse<List<WmStoreDTO>> listWmStore(Integer enterpriseId);

    ServiceResponse<WmStoreDTO> getWmStoreByWmMallStoreId(Integer wmMallStoreId);

    /**
     * 店铺配置
     * @Title: saveOrUpdateWmStoreConfig 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveOrUpdateWmStoreConfig(WmStoreConfigDTO dto);

    /**
     * 获取会员卡配置信息
     * @Title: getMemberCardConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 店铺主键
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.WmMemberCardConfigDTO> 
     */
    ServiceResponse<WmMemberCardConfigDTO> getMemberCardConfig(Integer wmMallStoreId);

    /**
     * 获取订单配置
     * @Title: getOrderConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.WmOrderConfigDTO> 
     */
    ServiceResponse<WmOrderConfigDTO> getOrderConfig(Integer wmMallStoreId);

    /**
     * 商品品牌
     * @Title: getGoodsConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 微盟店铺ID
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.wm.WmGoodsConfigDTO> 
     */
    ServiceResponse<WmGoodsConfigDTO> getGoodsConfig(Integer wmMallStoreId);

    /**
     * 会员卡域ID和企业查询的微盟店铺信息
     * @Title: getWmListByAppId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 商户ID
     * @param memberCardAreaIdList 会员卡域ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.wm.WmCouponPackageDTO>> 
     */
    ServiceResponse<List<WmCouponPackageDTO>> getWmListByMemberCardAreaId(Integer enterpriseId, List<Long> memberCardAreaIdList);

    /**
     * 会员卡域ID和企业查询的微盟店铺信息
     * @param enterpriseId
     * @param memberCardAreaId
     * @return
     */
    ServiceResponse<List<WmStoreInfoDTO>> listWmByMemberCardAreaId(Integer enterpriseId, Long memberCardAreaId);

    /**
     * 根据微盟主账号查询配置
     * @Title: getWmStoreConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 店铺主键ID 
     * @param wmStoreConfigType 店铺配置类型  WmStoreConfigTypeEnum 枚举类
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.wm.WmStoreConfigDTO> 
     */
    ServiceResponse<WmStoreConfigDTO> getWmStoreConfig(Integer wmMallStoreId, Integer wmStoreConfigType);

    /**
     * 查询店铺配置  key : 配置类型 WmStoreConfigTypeEnum 枚举类
     * @Title: getWmStoreConfigKey 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 店铺主键ID 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Set<String>> 
     */
    ServiceResponse<Set<String>> getWmStoreConfigKey(Integer wmMallStoreId);

    /**
     * 绑定卡券
     * @Title: saveCouponConfig 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> saveCouponConfig(WmStoreCouponConfigDTO dto);

    /**
     * 根据gic查询微盟绑定的卡券ID
     * @Title: getWmCouponId 
     * @Description:
     * @author guojuxing
     * @param couponId
     * @return  com.gic.enterprise.dto.wm.WmCouponDTO
     */
    ServiceResponse<List<WmCouponDTO>> getWmCouponId(Long couponId);

    /**
     * 分页查询卡券绑定记录
     * @Title: pageCouponConfig 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.wm.WmStoreCouponConfigDTO>> 
     */
    ServiceResponse<Page<WmStoreCouponConfigDTO>> pageCouponConfig(CouponConfigQO params);

    /**
     * 卡券删除消息通知，gic卡券删除
     * @Title: deleteWmGicCoupon 
     * @Description:
     * @author guojuxing
     * @param gicCouponId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> deleteWmGicCoupon(Long gicCouponId);
}
