package com.gic.enterprise.service.outer.impl.pay;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.*;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.entity.TabBillingOrder;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.entity.TabBillingRechargeRecord;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/20 15:36
 */
@Component("3")
public class OfflineStrategy implements PayStrategy {
    private Logger logger = LogManager.getLogger(OfflineStrategy.class);
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private BillingRechargeRecordService billingRechargeRecordService;
    @Autowired
    private BillingOrderService billingOrderService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    @Override
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(OutPayDTO outPayDTO) {

        // 判断该商户下是否有相同的订单号
        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(outPayDTO.getOrderSerialNumber(), PayTypeEnum.OFFLINE_PAY.getCode());
        if (payInfo != null) {
            return ServiceResponse.success(EntityUtil.changeEntityByJSON(BillingPayInfoDTO.class, payInfo));
        }
        BillingPayInfoDTO payInfoDTO = new BillingPayInfoDTO();
        payInfoDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
        payInfoDTO.setSerialNumber(CreateRandomUtils.createSerialNumber());
        payInfoDTO.setPayType(PayTypeEnum.OFFLINE_PAY.getCode());
        payInfoDTO.setTotalFeePaid(outPayDTO.getTotalFeePaid());
        payInfoDTO.setTimeStart(outPayDTO.getTimeStart());
        payInfoDTO.setResultCode(BillingPayResultEnum.WAIT.getCode());
        payInfoDTO.setInvoiceStatus(GlobalInfo.DATA_STATUS_DELETE);
        payInfoDTO.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        payInfoDTO.setCreateTime(new Date());
        payInfoDTO.setBuyType(outPayDTO.getBuyType());
        payInfoDTO.setOrderSerialNumber(outPayDTO.getOrderSerialNumber());
        //        payInfoDTO.setAuthCodeId(authCodeId);
        payInfoDTO.setRequestCode(outPayDTO.getRequestCode());
        payInfoDTO.setProductDescription(outPayDTO.getProductDescription());
        Integer line = billingPayInfoService.savePayInfo(payInfoDTO);
        if (line == 0) {
            logger.info("线下支付交易保存失败, {}", JSON.toJSONString(payInfoDTO));
            // 回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "线下支付交易保存失败");
        }
        return ServiceResponse.success(payInfoDTO);
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> confirmPay(OutPayDTO outPayDTO) {

        TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(outPayDTO.getEnterpriseId(), outPayDTO.getRequestCode() ,outPayDTO.getOrderSerialNumber());
        BillingPayInfoDTO billingPayInfoDTO = new BillingPayInfoDTO();

        if (payInfo == null) {
            String serialNumber = CreateRandomUtils.createSerialNumber();
            billingPayInfoDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
            billingPayInfoDTO.setSerialNumber(serialNumber);
            billingPayInfoDTO.setOrderSerialNumber(outPayDTO.getOrderSerialNumber());
            billingPayInfoDTO.setInvoiceStatus(GlobalInfo.DATA_STATUS_DELETE);
            billingPayInfoDTO.setPayType(PayTypeEnum.OFFLINE_PAY.getCode());
            billingPayInfoDTO.setTimeStart(DateUtil.formatString(new Date(), DateUtil.FORMAT_DATETIME_14));
            billingPayInfoDTO.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
            billingPayInfoDTO.setCreateTime(new Date());
            billingPayInfoDTO.setBuyType(outPayDTO.getBuyType());
            billingPayInfoDTO.setAuthCode(outPayDTO.getAuthCode());
            Integer payInfoId = billingPayInfoService.savePayInfo(billingPayInfoDTO);
            billingPayInfoDTO.setPayInfoId(payInfoId);
        } else {
            billingPayInfoDTO = EntityUtil.changeEntityByJSON(BillingPayInfoDTO.class, payInfo);
            // 更新交易信息
            ApprovalCallBackDTO dto = new ApprovalCallBackDTO();
            dto.setBuyType(payInfo.getBuyType());
            dto.setEnterpriseId(payInfo.getEnterpriseId());
            dto.setPaySerialNumber(payInfo.getSerialNumber());
            dto.setResultCode(BillingPayResultEnum.SUCCESS.getCode());
            Integer line = billingPayInfoService.updatePayInfoBySerialNumber(dto);
            if (line == 0) {
                logger.info("更新支付信息失败， {}", JSON.toJSONString(dto));
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "更新支付信息失败");
            }
        }
        billingPayInfoService.delOtherPayInfoExceptId(billingPayInfoDTO.getPayInfoId(), billingPayInfoDTO.getEnterpriseId(), billingPayInfoDTO.getOrderSerialNumber());
        // 发起审批
        initiateApproval(outPayDTO);
        return ServiceResponse.success(billingPayInfoDTO);

    }

    @Override
    public ServiceResponse<Boolean> queryAndUpdatePayStatus(Integer enterpriseId, String serialNumber) {
        logger.info("线下转账没有该方法 queryAndUpdatePayStatus");
        return ServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> callBack(ApprovalCallBackDTO resData) {
        logger.info("银行线下支付回调结果:{}", JSON.toJSONString(resData));



        String orderSerialNumber = resData.getOrderSerialNumber();
        TabBillingPayInfo billingPayInfo = billingPayInfoService.getByOrderSerialNumber(orderSerialNumber, PayTypeEnum.OFFLINE_PAY.getCode());
        resData.setPaySerialNumber(billingPayInfo.getSerialNumber());
        if (StringUtils.isBlank(billingPayInfo.getTimeEnd())) {
            // 写入支付结果
            Integer i = billingPayInfoService.updatePayInfoBySerialNumber(resData);
            switch (resData.getBuyType()) {
                case 1:
                    // 商户余额充值
                    billingRechargeRecordService.updateBySerialNumber(billingPayInfo.getEnterpriseId(), resData.getOrderSerialNumber(), resData.getTotalFee(), resData.getTotalFeePaid(), resData.getAuditStatus(), resData.getTimeEnd(), null);
                    sendRechargeSuccessSms(resData, billingPayInfo.getEnterpriseId());
                    sendApprovalSms(resData, "GICJFZX007");
                    break;
                case 2:
                    // 短信套餐包购买
                    this.billingOrderService.updateBySerialNumber(resData.getOrderSerialNumber(), resData.getTotalFeePaid(), resData.getAuditStatus(), resData.getTimeEnd());
                    sendOrderApprovalSms(resData, "GICJFZX008");
                    break;
                default:
                    break;
            }
        }else {
            logger.info("银行转账重复回调 orderSerialNumber:{}", resData.getOrderSerialNumber());
        }
        return ServiceResponse.success();
    }

    private void sendRechargeSuccessSms(ApprovalCallBackDTO resData, Integer enterpriseId) {
        UserDTO userDTO = userApiService.getUserByEnterpriseId(enterpriseId).getResult();
        sendSms(resData, userDTO, "GICJFZX005");
    }

    private void sendApprovalSms(ApprovalCallBackDTO resData, String code) {
        TabBillingRechargeRecord rechargeRecord = billingRechargeRecordService.getRechargeRecordBySerialNumber(resData.getOrderSerialNumber());
        Integer initiator = rechargeRecord.getInitiator();
        UserDTO userDTO = userApiService.getUserById(initiator).getResult();
        sendSms(resData, userDTO, code);
    }

    private void sendOrderApprovalSms(ApprovalCallBackDTO resData, String code) {
//        TabBillingRechargeRecord rechargeRecord = billingRechargeRecordService.getRechargeRecordBySerialNumber(resData.getOrderSerialNumber());
        TabBillingOrder order = billingOrderService.getOrderBySerialNumber(resData.getOrderSerialNumber());
        Integer initiator = order.getInitiatorUser();
        if (initiator != null) {
            UserDTO userDTO = userApiService.getUserById(initiator).getResult();
            sendSms(resData, userDTO, code);
        }
    }

    private void sendSms(ApprovalCallBackDTO resData, UserDTO userDTO, String code) {
        try {
            if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                userDTO.setPhoneAreaCode("+86");
            }
            EnterpriseDTO enterpriseDTO = enterpriseApiService.getEnterpriseById(userDTO.getEnterpriseId()).getResult();
            String[] smsArr = new String[]{enterpriseDTO.getEnterpriseName(), resData.getOrderSerialNumber(), BillingAuditStatusEnum.PASS.getCode().equals(resData.getAuditStatus()) ? "成功" : "失败"};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(code,
                    userDTO.getEnterpriseId(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
            if (!smsSendResult.isSuccess()) {
                logger.warn(smsSendResult.getMessage());
            }
        } catch (Exception e) {
            logger.info("发送短信失败", e);
        }
    }


    private void initiateApproval(OutPayDTO outPayDTO) {
        TransferAccountsApprovalDTO approvalDTO = new TransferAccountsApprovalDTO();
        approvalDTO.setOrderNumber(outPayDTO.getOrderSerialNumber());
        // gic
        approvalDTO.setPlatformType(1);
        if (outPayDTO.getInitiatorUser() != null) {
            UserDTO userDTO = userApiService.getUserById(outPayDTO.getInitiatorUser()).getResult();
            if (userDTO != null) {
                approvalDTO.setInitiatorType(userDTO.getLoginType() == 0 ?
                        BillingInitiatorTypeEnum.ENTERPRISE.getCode() : BillingInitiatorTypeEnum.OPERATION.getCode());
            }
        }
        approvalDTO.setInitiatorName(outPayDTO.getInitiatorName());
        approvalDTO.setPlannedAmount(outPayDTO.getTotalFee());
        approvalDTO.setVerifyCode(outPayDTO.getAuthCode());
        approvalDTO.setOrderType(outPayDTO.getBuyType());
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(outPayDTO.getEnterpriseId());
        approvalDTO.setEnterpriseId(outPayDTO.getEnterpriseId());
        approvalDTO.setCompanyName(enterprise.getCompanyName());
        approvalDTO.setEnterpriseName(enterprise.getEnterpriseName());
        approvalDTO.setLogo(enterprise.getLogo());
//        approvalDTO.setCall
//com.gic.enterprise.service.BillingPayInfoApiService.offlineCallBack()
        logger.info("调用财务模块接口:{}", JSON.toJSONString(approvalDTO));

        ServiceResponse<Void> serviceResponse = new ServiceResponse<>();
        if (approvalDTO.getOrderType() == BuyTypeEnum.BALANCE_RECHARGE.getCode()) {
            serviceResponse = transferAccountsApprovalApiService.rechargeInitiateApproval(approvalDTO);
        } else if (approvalDTO.getOrderType() == BuyTypeEnum.BUY_SHORT_MESSAGE_PACKAGE.getCode()) {
            serviceResponse = transferAccountsApprovalApiService.smsInitiateApproval(approvalDTO);
        }
        logger.info("调用财务模块返回:{},{}", serviceResponse.isSuccess(), serviceResponse.getMessage());
    }

}
