package com.gic.enterprise.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.BillingCouponCardDTO;
import com.gic.enterprise.dto.BillingOrderDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.BillingSmsRecordDTO;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.billing.BillingCouponCardVO;
import com.gic.enterprise.web.vo.billing.OrderApprovalVO;
import com.gic.enterprise.web.vo.billing.OrderDetailVO;
import com.gic.enterprise.web.vo.billing.SmsPackageOrderItemVO;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class BillingOrderController {
    private static final Logger log = LogManager.getLogger(BillingOrderController.class);

    @Autowired
    private BillingOrderApiService billingOrderApiService;
    @Autowired
    private BillingPayInfoApiService billingPayInfoApiService;
    @Autowired
    private BillingCouponCardApiService billingCouponCardApiService;
    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;
    @Autowired
    private BillingSmsRecordApiService billingSmsRecordApiService;

    @RequestMapping("page-order")
    public RestResponse pageOrder(PageQO pageQO, String search, Integer status, String startDate, String endDate){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<BillingOrderDTO>> pageServiceResponse = this.billingOrderApiService.listOrder(startDate,
                endDate, status, search, pageQO.getCurrentPage(), pageQO.getPageSize(), enterpriseId);
        if(pageServiceResponse.isSuccess()){
            return RestResponse.success(pageServiceResponse.getResult());
        }
        return EnterpriseRestResponse.failure(pageServiceResponse);
    }

    @RequestMapping("order-detail")
    public RestResponse orderDetail(Integer orderId){
        ServiceResponse<BillingOrderDTO> order = this.billingOrderApiService.getOrderById(orderId);
        if(order.isSuccess()){
            BillingOrderDTO dto = order.getResult();
            if(dto != null){
                OrderDetailVO vo = new OrderDetailVO();
                vo.setSerialNumber(dto.getSerialNumber());
                vo.setCreateTime(dto.getCreateTime());
                vo.setPayStatus(dto.getOrderStatus());
                vo.setPayType(dto.getPayType());
                vo.setTotalFee(dto.getTotalFee());
                vo.setTotalFeePaid(dto.getTotalFeePaid());
//                ServiceResponse<BillingPayInfoDTO> payInfoDetail = this.billingPayInfoApiService.getPayInfoDetail(dto.getPayInfoId());
                ServiceResponse<BillingPayInfoDTO> payInfoDetail = this.billingPayInfoApiService.getPayInfo(dto.getEnterpriseId(), dto.getSerialNumber(), null);
                if(payInfoDetail.isSuccess()){
                    BillingPayInfoDTO payInfoDTO = payInfoDetail.getResult();
                    if(payInfoDTO != null){
                        vo.setPayTime(payInfoDTO.getUpdateTime());
                        vo.setPaySerialNumber(payInfoDTO.getSerialNumber());
                        vo.setInvoiceStatus(payInfoDTO.getInvoiceStatus());
                    }
                }
                vo.setOrderItemVO(JSON.parseObject(dto.getItemJson(), SmsPackageOrderItemVO.class));
                ServiceResponse<BillingCouponCardDTO> billingCouponCard = this.billingCouponCardApiService.getBillingCouponCard(dto.getCouponCardId());
                if(billingCouponCard.isSuccess()){
                    BillingCouponCardDTO couponCardDTO = billingCouponCard.getResult();
                    if(couponCardDTO != null){
                        vo.setCouponCardVO(EntityUtil.changeEntityByJSON(BillingCouponCardVO.class, couponCardDTO));
                    }
                }
                if(dto.getPayType() != null && dto.getPayType() == PayTypeEnum.OFFLINE_PAY.getCode()){
                    ServiceResponse<TransferAccountsApprovalDTO> byOrderNumber = this.transferAccountsApprovalApiService.
                            getByOrderNumber(dto.getSerialNumber());
                    if(byOrderNumber.isSuccess()){
                        TransferAccountsApprovalDTO approvalDTO = byOrderNumber.getResult();
                        if(approvalDTO != null){
                            vo.setApprovalVO(EntityUtil.changeEntityByJSON(OrderApprovalVO.class, approvalDTO));
                        }
                    }
                }
                return RestResponse.success(vo);
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("/cancel-order-approval")
    public RestResponse cancelRechargeApproval(Integer orderId) {
        ServiceResponse<Integer> serviceResponse = this.billingOrderApiService.cancelOrderApproval(orderId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-sms-package-record")
    public RestResponse listSmsPackageRecord(String search, String startDate, String endDate, Integer smsPackageId, int currentPage, int pageSize) {
        ServiceResponse<Page<BillingSmsRecordDTO>> resp = billingSmsRecordApiService.pageSmspackageRecord(search, startDate, endDate, smsPackageId, currentPage, pageSize);
        return ResultControllerUtils.commonResult(resp);
    }


}
